from _mutate import XMLP_Mutator, XMLP_Mutated
import _mutate
import copy

class _EmptyClass: pass

class mutate_builtin_wrapper(XMLP_Mutator):

    def __init__(self):
        XMLP_Mutator.__init__(self,None,'builtin_wrapper')

    def mutate(self,obj):
        wrap = _EmptyClass()
        # this is the "gotcha" that makes us turn off deepcopy
        # for wrappers -- we've already saved obj/id(obj), and
        # the WRAPPER will be referenced as that id, NOT obj.
        # hence, when the pickler gets to __toplevel__ it'll
        # ref it to the WRAPPER, which is meaningless, so we blow up.
        # IAW, cyclic references ARE supposed to work, but what we've
        # done is substitute a fake obj for a real one under the same id.
        wrap.__toplevel__ = obj
        return XMLP_Mutated(wrap)

    def unmutate(self,mobj):
        return mobj.obj.__toplevel__
    
_mutate.add_mutator(mutate_builtin_wrapper())

# We pickle array.array() as type "array" and Numeric.array as
# type "Numpy_array" (this is really what earlier xml_pickles did,
# except you had to use EITHER array.array() or Numeric.array() -
# you couldn't mix them (in fact, you couldn't pickle array.array()
# types if Numeric was installed).

import array

#-- array.array --
class mutate_array(XMLP_Mutator):

    def __init__(self):
        XMLP_Mutator.__init__(self,array.ArrayType,'array',0)
        
    def mutate(self,obj):
        list = []
        for item in obj:
            list.append(item)
        return XMLP_Mutated(list)

    def unmutate(self,mobj):
        obj = mobj.obj
        as_int = 1
        for item in obj:
            if type(item) == type(1.0):
                as_int = 0
        if as_int:
            return array.array('b',obj)
        else:
            return array.array('f',obj)

_mutate.add_mutator(mutate_array())

#-- Numeric.array --
try:
    import Numeric
    HAS_NUMERIC = 1
except:
    HAS_NUMERIC = 0

class mutate_numpy(XMLP_Mutator):

    def __init__(self):
        # note, Numeric.ArrayType != array.ArrayType, which is good :-)
        XMLP_Mutator.__init__(self,Numeric.ArrayType,'NumPy_array',0)
        
    def mutate(self,obj):
        list = []
        for item in obj:
            list.append(item)
        return XMLP_Mutated(list)

    def unmutate(self,mobj):
        return Numeric.array(mobj.obj)

if HAS_NUMERIC:
    _mutate.add_mutator(mutate_numpy())

#-- SREs --

# save the RE pattern in the element body

import re
SRE_Pattern_type = type(re.compile(''))

class mutate_sre(XMLP_Mutator):

    def __init__(self):
        XMLP_Mutator.__init__(self,SRE_Pattern_type,'SRE',paranoia=0,
                              in_body=1)
        
    def mutate(self,obj):
        return XMLP_Mutated(obj.pattern)

    def unmutate(self,mobj):
        return re.compile(mobj.obj)

_mutate.add_mutator(mutate_sre())

#-- rawpickles --

# save the pickle in the element body

try:    import cPickle as pickle
except: import pickle

class mutate_rawpickle(XMLP_Mutator):

    def __init__(self):
        XMLP_Mutator.__init__(self,None,'rawpickle',0)
        
    def mutate(self,obj): return XMLP_Mutated(pickle.dumps(obj))
    def unmutate(self,mobj): return pickle.loads(str(mobj.obj))

_mutate.add_mutator(mutate_rawpickle())

#-- mx.DateTime --

# see test_mutators.py for an alternate way to pickle these

try:
    import mx.DateTime
    mxDateTime_type = type(mx.DateTime.localtime())
except:
    mxDateTime_type = None

class mutate_mxdatetime(XMLP_Mutator):

    def __init__(self):
        XMLP_Mutator.__init__(self,mxDateTime_type,'mxDateTime',
                              paranoia=0,in_body=1)
        
    def mutate(self,obj):
        # (I avoided using strftime(), for portability reasons.)
        # Pickle seconds as a float to save full precision.
        s = "YMD = %d/%d/%d, HMS = %d:%d:%f" % \
            (obj.year,obj.month,obj.day,\
             obj.hour,obj.minute,obj.second)
        return XMLP_Mutated(s)

    def unmutate(self,mobj):
        obj = mobj.obj
        # is this forgiving enough? :-)
        fmt = 'YMD\s*=\s*([0-9]+)\s*/\s*([0-9]+)\s*/\s*([0-9]+)\s*,\s*'
        fmt += 'HMS\s*=\s*([0-9]+)\s*:\s*([0-9]+)\s*:\s*([0-9\.]+)'
        m = re.match(fmt,obj)
        return apply(mx.DateTime.DateTime,map(float,m.groups()))

if mxDateTime_type is not None:
    _mutate.add_mutator(mutate_mxdatetime())
